@@ -2,6 +2,8 @@
 import os
 from urlparse import urlparse
 
+from paste.deploy.converters import asbool
+
 import pylons
 from sqlalchemy import engine_from_config
 from pylons import config
@@ -13,8 +15,8 @@ import ckan.lib.app_globals as app_globals
 import ckan.lib.helpers
 from ckan.config.routing import make_map
 from ckan import model
+from ckan import plugins
 
-import plugins
 
 
 def load_environment(global_conf, app_conf):
@@ -69,12 +71,12 @@ def load_environment(global_conf, app_conf):
     # any Pylons config options)    
 
     # Setup the SQLAlchemy database engine
-    engine = engine_from_config(config, 'sqlalchemy.')
+    engine = engine_from_config(config, 'sqlalchemy.', pool_threadlocal=True)
     model.init_model(engine)
-        
-    if bool(config.get('ckan.build_search_index_synchronously', True)):
-        import ckan.lib.search as search
-        search.setup_synchronous_indexing()
-    else:
-        import ckan.lib.async_notifier as notf
-
+    
+    from ckan.plugins import ExtensionPoint
+    from ckan.plugins.interfaces import IConfigurable
+    
+    for service in ExtensionPoint(IConfigurable):
+        service.configure(config)
+    
