@@ -1,13 +1,17 @@
 import logging
 import urlparse
 
+from sqlalchemy.orm import eagerload_all
+from sqlalchemy import or_
 import genshi
-from pylons import config
+from pylons import config, cache
 from pylons.i18n import get_lang, _
 
 from ckan.lib.base import *
-from ckan.lib.search import query_for, QueryOptions
+from ckan.lib.search import query_for, QueryOptions, SearchError
+from ckan.lib.cache import proxy_cache
 from ckan.lib.package_saver import PackageSaver, ValidationException
+from ckan.plugins import ExtensionPoint, IPackageController
 import ckan.forms
 import ckan.authz
 import ckan.rating
@@ -17,14 +21,18 @@ logger = logging.getLogger('ckan.controllers')
 
 class PackageController(BaseController):
     authorizer = ckan.authz.Authorizer()
+    extensions = ExtensionPoint(IPackageController)
 
     def index(self):
         query = ckan.authz.Authorizer().authorized_query(c.user, model.Package)
         c.package_count = query.count()
         return render('package/index.html')
 
+    @proxy_cache()
     def list(self):
         query = ckan.authz.Authorizer().authorized_query(c.user, model.Package)
+        query = query.options(eagerload_all('package_tags.tag'))
+        query = query.options(eagerload_all('package_resources_all'))
         c.page = h.AlphaPage(
             collection=query,
             page=request.params.get('page', 'A'),
@@ -38,63 +46,84 @@ class PackageController(BaseController):
         c.open_only = request.params.get('open_only')
         c.downloadable_only = request.params.get('downloadable_only')
         if c.q:
+            c.query_error = False
             page = int(request.params.get('page', 1))
             limit = 20
             query = query_for(model.Package)
-            query.run(query=c.q,
-                      limit=limit,
-                      offset=(page-1)*limit,
-                      return_objects=True,
-                      filter_by_openness=c.open_only,
-                      filter_by_downloadable=c.downloadable_only,
-                      username=c.user)
+            try:
+                query.run(query=c.q,
+                          limit=limit,
+                          offset=(page-1)*limit,
+                          return_objects=True,
+                          filter_by_openness=c.open_only,
+                          filter_by_downloadable=c.downloadable_only,
+                          username=c.user)
             
-            c.page = h.Page(
-                collection=query.results,
-                page=page,
-                #items=query.results,
-                item_count=query.count,
-                items_per_page=limit
-            )
-            c.page.items = query.results
+                c.page = h.Page(
+                    collection=query.results,
+                    page=page,
+                    item_count=query.count,
+                    items_per_page=limit
+                )
+                c.page.items = query.results
+            except SearchError, se:
+                c.query_error = True
+                c.page = h.Page(collection=[])
             
             # tag search
             c.tag_limit = 25
             query = query_for('tag', backend='sql')
-            query.run(query=c.q,
-                      return_objects=True,
-                      limit=c.tag_limit,
-                      username=c.user)
-            c.tags = query.results
-            c.tags_count = query.count
+            try:
+                query.run(query=c.q,
+                          return_objects=True,
+                          limit=c.tag_limit,
+                          username=c.user)
+                c.tags = query.results
+                c.tags_count = query.count
+            except SearchError, se:
+                c.tags = []
+                c.tags_count = 0
 
         return render('package/search.html')
 
+    @staticmethod
+    def _pkg_cache_key(pkg):
+        # note: we need pkg.id in addition to pkg.revision.id because a
+        # revision may have more than one package in it.
+        return str(hash((pkg.id, pkg.latest_related_revision.id, c.user, pkg.get_average_rating())))
+        
+    def _clear_pkg_cache(self, pkg):
+        read_cache = cache.get_cache('package/read.html', type='dbm')
+        read_cache.remove_value(self._pkg_cache_key(pkg))
+
+    @proxy_cache()
     def read(self, id):
-        pkg = model.Package.get(id)
-        if pkg is None:
+        c.pkg = model.Package.get(id)
+        if c.pkg is None:
             abort(404, gettext('Package not found'))
         
+        cache_key = self._pkg_cache_key(c.pkg)
+        etag_cache(cache_key)
+        
         # used by disqus plugin
-        c.current_package_id = pkg.id
+        c.current_package_id = c.pkg.id
         
         if config.get('rdf_packages'):
             accept_headers = request.headers.get('Accept', '')
             if 'application/rdf+xml' in accept_headers and \
                    not 'text/html' in accept_headers:
-                rdf_url = '%s%s' % (config['rdf_packages'], pkg.name)
+                rdf_url = '%s%s' % (config['rdf_packages'], c.pkg.name)
                 redirect(rdf_url, code=303)
 
-        auth_for_read = self.authorizer.am_authorized(c, model.Action.READ, pkg)
+        auth_for_read = self.authorizer.am_authorized(c, model.Action.READ, c.pkg)
         if not auth_for_read:
             abort(401, str(gettext('Unauthorized to read package %s') % id))
+        
+        for item in self.extensions:
+            item.read(c.pkg)
 
-        c.auth_for_authz = self.authorizer.am_authorized(c, model.Action.EDIT_PERMISSIONS, pkg)
-        c.auth_for_edit = self.authorizer.am_authorized(c, model.Action.EDIT, pkg)
-        c.auth_for_change_state = self.authorizer.am_authorized(c, model.Action.CHANGE_STATE, pkg)
-
-        PackageSaver().render_package(pkg)
-        return render('package/read.html') 
+        PackageSaver().render_package(c.pkg)
+        return render('package/read.html')
 
     def history(self, id):
         if 'diff' in request.params or 'selected1' in request.params:
@@ -108,6 +137,7 @@ class PackageController(BaseController):
                     id = request.params.getone('pkg_name')
                 c.error = _('Select two revisions before doing the comparison.')
             else:
+                params['diff_entity'] = 'package'
                 h.redirect_to(controller='revision', action='diff', **params)
 
         c.pkg = model.Package.get(id)
@@ -157,21 +187,24 @@ class PackageController(BaseController):
 
     def new(self):
         c.error = ''
-
+        api_url = config.get('ckan.api_url', '/').rstrip('/')
+        c.package_create_slug_api_url = api_url+h.url_for(controller='apiv2/package', action='create_slug')
         is_admin = self.authorizer.is_sysadmin(c.user)
-
-        fs = ckan.forms.registry.get_fieldset(is_admin=is_admin,
-                         package_form=request.params.get('package_form'))
+        # Check access control for user to create a package.
+        auth_for_create = self.authorizer.am_authorized(c, model.Action.PACKAGE_CREATE, model.System())
+        if not auth_for_create:
+            abort(401, str(gettext('Unauthorized to create a package')))
+        # Get the name of the package form.
+        fs = self._get_package_fieldset(is_admin=is_admin)
         if 'save' in request.params or 'preview' in request.params:
             if not request.params.has_key('log_message'):
                 abort(400, ('Missing parameter: log_message'))
             log_message = request.params['log_message']
-
         record = model.Package
         if request.params.has_key('save'):
             fs = fs.bind(record, data=dict(request.params) or None, session=model.Session)
             try:
-                PackageSaver().commit_pkg(fs, None, None, log_message, c.author)
+                PackageSaver().commit_pkg(fs, None, None, log_message, c.author, client=c)
                 pkgname = fs.name.value
 
                 pkg = model.Package.by_name(pkgname)
@@ -181,6 +214,8 @@ class PackageController(BaseController):
                     if user:
                         admins = [user]
                 model.setup_default_user_roles(pkg, admins)
+                for item in self.extensions:
+                    item.create(pkg)
                 model.repo.commit_and_remove()
 
                 self._form_save_redirect(pkgname, 'new')
@@ -205,12 +240,14 @@ class PackageController(BaseController):
             fs = fs.bind(model.Package, data=data, session=model.Session)
         else:
             fs = fs.bind(session=model.Session)
+        #if 'preview' in request.params:
+        #    c.preview = ' '
         c.form = self._render_edit_form(fs, request.params, clear_session=True)
         if 'preview' in request.params:
             try:
                 PackageSaver().render_preview(fs, id, record.id,
                                               log_message=log_message,
-                                              author=c.author)
+                                              author=c.author, client=c)
                 c.preview = h.literal(render('package/read_core.html'))
             except ValidationException, error:
                 fs = error.args[0]
@@ -222,18 +259,15 @@ class PackageController(BaseController):
     def edit(self, id=None): # allow id=None to allow posting
         # TODO: refactor to avoid duplication between here and new
         c.error = ''
-
-        pkg = model.Package.get(id)
+        c.pkg = pkg = model.Package.get(id)
         if pkg is None:
             abort(404, '404 Not Found')
         am_authz = self.authorizer.am_authorized(c, model.Action.EDIT, pkg)
         if not am_authz:
             abort(401, str(gettext('User %r not authorized to edit %s') % (c.user, id)))
 
-        c.auth_for_change_state = self.authorizer.am_authorized(c, model.Action.CHANGE_STATE, pkg)
-        fs = ckan.forms.registry.get_fieldset(is_admin=c.auth_for_change_state,
-                       package_form=request.params.get('package_form'))
-
+        auth_for_change_state = self.authorizer.am_authorized(c, model.Action.CHANGE_STATE, pkg)
+        fs = self._get_package_fieldset(is_admin=auth_for_change_state)
         if 'save' in request.params or 'preview' in request.params:
             if not request.params.has_key('log_message'):
                 abort(400, ('Missing parameter: log_message'))
@@ -255,7 +289,9 @@ class PackageController(BaseController):
                                           # multidict which is read only
             fs = fs.bind(pkg, data=params or None)
             try:
-                PackageSaver().commit_pkg(fs, id, pkg.id, log_message, c.author)
+                for item in self.extensions:
+                    item.edit(fs.model)
+                PackageSaver().commit_pkg(fs, id, pkg.id, log_message, c.author, client=c)
                 # do not use package name from id, as it may have been edited
                 pkgname = fs.name.value
                 self._form_save_redirect(pkgname, 'edit')
@@ -275,7 +311,7 @@ class PackageController(BaseController):
             try:
                 PackageSaver().render_preview(fs, id, pkg.id,
                                               log_message=log_message,
-                                              author=c.author)
+                                              author=c.author, client=c)
                 c.pkgname = fs.name.value
                 c.pkgtitle = fs.title.value
                 read_core_html = render('package/read_core.html') #utf8 format
@@ -339,7 +375,10 @@ class PackageController(BaseController):
                 raise
             # now do new roles
             newrole_user_id = request.params.get('PackageRole--user_id')
-            if newrole_user_id != '__null_value__':
+            newrole_authzgroup_id = request.params.get('PackageRole--authorized_group_id')
+            if newrole_user_id != '__null_value__' and newrole_authzgroup_id != '__null_value__':
+                c.message = _(u'Please select either a user or an authorization group, not both.')
+            elif newrole_user_id != '__null_value__':
                 user = model.Session.query(model.User).get(newrole_user_id)
                 # TODO: chech user is not None (should go in validation ...)
                 role = request.params.get('PackageRole--role')
@@ -348,18 +387,42 @@ class PackageController(BaseController):
                 # With FA no way to get new PackageRole back to set package attribute
                 # new_roles = ckan.forms.new_roles_fs.bind(model.PackageRole, data=params or None)
                 # new_roles.sync()
+                for item in self.extensions:
+                    item.authz_add_role(newpkgrole)
                 model.repo.commit_and_remove()
                 c.message = _(u'Added role \'%s\' for user \'%s\'') % (
                     newpkgrole.role,
-                    newpkgrole.user.name)
+                    newpkgrole.user.display_name)
+            elif newrole_authzgroup_id != '__null_value__':
+                authzgroup = model.Session.query(model.AuthorizationGroup).get(newrole_authzgroup_id)
+                # TODO: chech user is not None (should go in validation ...)
+                role = request.params.get('PackageRole--role')
+                newpkgrole = model.PackageRole(authorized_group=authzgroup, 
+                        package=pkg, role=role)
+                # With FA no way to get new GroupRole back to set group attribute
+                # new_roles = ckan.forms.new_roles_fs.bind(model.GroupRole, data=params or None)
+                # new_roles.sync()
+                for item in self.extensions:
+                    item.authz_add_role(newpkgrole)
+                model.Session.commit()
+                model.Session.remove()
+                c.message = _(u'Added role \'%s\' for authorization group \'%s\'') % (
+                    newpkgrole.role,
+                    newpkgrole.authorized_group.name)
         elif 'role_to_delete' in request.params:
             pkgrole_id = request.params['role_to_delete']
             pkgrole = model.Session.query(model.PackageRole).get(pkgrole_id)
             if pkgrole is None:
                 c.error = _(u'Error: No role found with that id')
             else:
-                c.message = _(u'Deleted role \'%s\' for user \'%s\'') % (pkgrole.role,
-                        pkgrole.user.name)
+                for item in self.extensions:
+                    item.authz_remove_role(pkgrole)
+                if pkgrole.user:
+                    c.message = _(u'Deleted role \'%s\' for user \'%s\'') % \
+                                (pkgrole.role, pkgrole.user.display_name)
+                elif pkgrole.authorized_group:
+                    c.message = _(u'Deleted role \'%s\' for authorization group \'%s\'') % \
+                                (pkgrole.role, pkgrole.authorized_group.name)
                 pkgrole.purge()
                 model.repo.commit_and_remove()
 
@@ -367,7 +430,8 @@ class PackageController(BaseController):
         c.pkg = model.Package.get(id)
         fs = ckan.forms.get_authz_fieldset('package_authz_fs').bind(c.pkg.roles)
         c.form = fs.render()
-        c.new_roles_form = ckan.forms.get_authz_fieldset('new_package_roles_fs').render()
+        c.new_roles_form = \
+            ckan.forms.get_authz_fieldset('new_package_roles_fs').render()
         return render('package/authz.html')
 
     def rate(self, id):
@@ -375,22 +439,30 @@ class PackageController(BaseController):
         package = model.Package.get(package_name)
         if package is None:
             abort(404, gettext('404 Package Not Found'))
+        self._clear_pkg_cache(package)
         rating = request.params.get('rating', '')
         if rating:
             try:
                 ckan.rating.set_my_rating(c, package, rating)
             except ckan.rating.RatingValueException, e:
                 abort(400, gettext('Rating value invalid'))
-        h.redirect_to(controller='package', action='read', id=package_name)
+        h.redirect_to(controller='package', action='read', id=package_name, rating=str(rating))
 
     def autocomplete(self):
+        q = unicode(request.params.get('q', ''))
+        if not len(q): 
+            return ''
         pkg_list = []
+        like_q = u"%s%%" % q
         pkg_query = ckan.authz.Authorizer().authorized_query(c.user, model.Package)
+        pkg_query = pkg_query.filter(or_(model.Package.name.ilike(like_q),
+                                         model.Package.title.ilike(like_q)))
+        pkg_query = pkg_query.limit(10)
         for pkg in pkg_query:
-            pkg_list.extend([
-                '%s (%s)|%s' % (pkg.title, pkg.name, pkg.id),
-                '%s|%s' % (pkg.name, pkg.id),
-                ])
+            if pkg.name.lower().startswith(q.lower()):
+                pkg_list.append('%s|%s' % (pkg.name, pkg.name))
+            else:
+                pkg_list.append('%s (%s)|%s' % (pkg.title.replace('|', ' '), pkg.name, pkg.name))
         return '\n'.join(pkg_list)
 
     def _render_edit_form(self, fs, params={}, clear_session=False):
@@ -398,6 +470,7 @@ class PackageController(BaseController):
         c.log_message = params.get('log_message', '')
         # expunge everything from session so we don't have any problematic
         # saves (this gets called on validation exceptions a lot)
+        # Todo: Explain why there are 'saves' when rendering a form.
         if clear_session:
             model.Session.clear()
         edit_form_html = fs.render()
