@@ -1,8 +1,9 @@
 from pylons.i18n import get_lang
 
 from ckan.lib.base import *
+from ckan.lib.helpers import Page
 import ckan.authz
-from datetime import datetime
+from datetime import datetime, timedelta
 
 class RevisionController(BaseController):
 
@@ -20,20 +21,21 @@ class RevisionController(BaseController):
                 description=_(u'Recent changes to the CKAN repository.'),
                 language=unicode(get_lang()),
             )
-            select_results = model.repo.history().all()
-            for revision in select_results:
-                if not revision.id and revision.number:
-                    continue
-                try:
-                    dayHorizon = int(request.params.get('days'))
-                except:
-                    dayHorizon = 30
-                try:
-                    dayAge = (datetime.now() - revision.timestamp).days
-                except:
-                    dayAge = 0
-                if dayAge >= dayHorizon:
-                    break
+            # TODO: make this configurable?
+            # we do not want the system to fall over!
+            maxresults = 200
+            try:
+                dayHorizon = int(request.params.get('days', 5))
+            except:
+                dayHorizon = 5
+            ourtimedelta = timedelta(days=-dayHorizon)
+            since_when = datetime.now() + ourtimedelta
+            revision_query = model.repo.history()
+            revision_query = revision_query.filter(
+                    model.Revision.timestamp>=since_when).filter(
+                    model.Revision.id!=None)
+            revision_query = revision_query.limit(maxresults)
+            for revision in revision_query:
                 package_indications = []
                 revision_changes = model.repo.list_changes(revision)
                 package_resource_revisions = revision_changes[model.PackageResource]
@@ -85,20 +87,21 @@ class RevisionController(BaseController):
             return feed.writeString('utf-8')
         else:
             c.show_purge_links = self._has_purge_permissions()
-                        
-            from ckan.lib.helpers import Page
-
+            query = model.Session.query(model.Revision)
             c.page = Page(
-                collection=model.Session.query(model.Revision),
+                collection=query,
                 page=request.params.get('page', 1),
-                items_per_page=50
+                items_per_page=20
             )
-            
             return render('revision/list.html')
 
     def read(self, id=None):
         if id is None:
             h.redirect_to(controller='revision', action='list')
+        
+        cache_key = str(hash(id))
+        etag_cache(cache_key)    
+        
         c.revision = model.Session.query(model.Revision).get(id)
         if c.revision is None:
             abort(404)
@@ -106,20 +109,30 @@ class RevisionController(BaseController):
         c.packages = [ pkg.continuity for pkg in pkgs ]
         pkgtags = model.Session.query(model.PackageTagRevision).filter_by(revision=c.revision)
         c.pkgtags = [ pkgtag.continuity for pkgtag in pkgtags ]
-        return render('revision/read.html')
+        grps = model.Session.query(model.GroupRevision).filter_by(revision=c.revision)
+        c.groups = [ grp.continuity for grp in grps ]
+        return render('revision/read.html', cache_key=cache_key)
 
     def diff(self, id=None):
         if 'diff' not in request.params or 'oldid' not in request.params:
             abort(400)
-        pkg = model.Package.by_name(id)
         c.revision_from = model.Session.query(model.Revision).get(
             request.params.getone('oldid'))
         c.revision_to = model.Session.query(model.Revision).get(
             request.params.getone('diff'))
-        diff = pkg.diff(c.revision_to, c.revision_from)
+        
+        c.diff_entity = request.params.get('diff_entity')
+        if c.diff_entity == 'package':
+            c.pkg = model.Package.by_name(id)
+            diff = c.pkg.diff(c.revision_to, c.revision_from)
+        elif c.diff_entity == 'group':
+            c.group = model.Group.by_name(id)
+            diff = c.group.diff(c.revision_to, c.revision_from)
+        else:
+            abort(400)
+        
         c.diff = diff.items()
         c.diff.sort()
-        c.pkg = pkg
         return render('revision/diff.html')
 
     def _has_purge_permissions(self):
