@@ -4,10 +4,11 @@ from formalchemy import helpers as fa_h
 import formalchemy
 import genshi
 from pylons.templating import render_genshi as render
-from pylons import c
+from pylons import c, config
 from pylons.i18n import _, ungettext, N_, gettext
 
 from ckan.lib.helpers import literal
+from ckan.authz import Authorizer
 import ckan.model as model
 import ckan.lib.helpers as h
 import ckan.lib.field_types as field_types
@@ -21,7 +22,12 @@ def name_validator(val, field=None):
         raise formalchemy.ValidationError(_('Name must be at least %s characters long') % min_length)
     if not name_match.match(val):
         raise formalchemy.ValidationError(_('Name must be purely lowercase alphanumeric (ascii) characters and these symbols: -_'))
-        
+
+def package_exists(val):
+    if model.Session.query(model.Package).autoflush(False).filter_by(name=val).count():
+        return True
+    return False
+
 def package_name_validator(val, field=None):
     name_validator(val, field)
     # we disable autoflush here since may get used in package preview
@@ -38,20 +44,41 @@ def group_name_validator(val, field=None):
         if group != field.parent.model:
             raise formalchemy.ValidationError(_('Group name already exists in database'))
 
-def field_readonly_renderer(key, value, newline_reqd=True):
+def harvest_source_url_validator(val, field=None):
+    if not val.strip().startswith('http://'):
+        raise formalchemy.ValidationError(_('Harvest source URL is invalid (must start with "http://").'))
+
+
+def field_readonly_renderer(key, value, newline_reqd=False):
     if value is None:
         value = ''
-    key = key.capitalize().replace('_', ' ').replace('-', ' ')
-    if key in ('Url', 'Download url', 'Taxonomy url'):
-        key = key.replace(u'Url', u'URL')
-        key = key.replace(u'url', u'URL')
-        value = literal('<a href="%s">%s</a>') % (value, value)
-#        value = '<a href="%s">%s</a>' % (value, value)
-    html = literal('<strong>%s:</strong> %s') % (key, value)
+    html = literal('<p>%s</p>') % value
     if newline_reqd:
         html += literal('<br/>')
     return html
 
+class DateTimeFieldRenderer(formalchemy.fields.DateTimeFieldRenderer):
+    def render_readonly(self, **kwargs):
+        return field_readonly_renderer(self.field.key,
+                formalchemy.fields.DateTimeFieldRenderer.render_readonly(self, **kwargs))
+
+class CheckboxFieldRenderer(formalchemy.fields.CheckBoxFieldRenderer):
+    def render_readonly(self, **kwargs):
+        value = u'yes' if self.raw_value else u'no'
+        return field_readonly_renderer(self.field.key, value)
+
+class TextRenderer(formalchemy.fields.TextFieldRenderer):
+    def render_readonly(self, **kwargs):
+        return field_readonly_renderer(self.field.key, self.raw_value)
+
+class SelectFieldRenderer(formalchemy.fields.SelectFieldRenderer):
+    def render_readonly(self, **kwargs):
+        return field_readonly_renderer(self.field.key,
+                formalchemy.fields.SelectFieldRenderer.render_readonly(self, **kwargs))
+
+class TextAreaRenderer(formalchemy.fields.TextAreaFieldRenderer):
+    def render_readonly(self, **kwargs):
+        return field_readonly_renderer(self.field.key, self.raw_value)
 
 class TextExtraRenderer(formalchemy.fields.TextFieldRenderer):
     def _get_value(self):
@@ -385,7 +412,9 @@ class TagField(ConfiguredField):
             pkg_id = self.field.parent.model.id or ''
             kwargs['value'] = self._tags_string()
             kwargs['size'] = 60
-            kwargs['data-tagcomplete-url'] = h.url_for(controller='tag', action='autocomplete', id=None)
+            api_url = config.get('ckan.api_url', '/').rstrip('/')
+            tagcomplete_url = api_url+h.url_for(controller='apiv2/package', action='autocomplete', id=None)
+            kwargs['data-tagcomplete-url'] = tagcomplete_url
             kwargs['data-tagcomplete-queryparam'] = 'incomplete'
             kwargs['class'] = 'long tagComplete'
             html = literal(fa_h.text_field(self.name, **kwargs))
@@ -526,7 +555,7 @@ class ExtrasField(ConfiguredField):
             # ('Package-1-extras-newfield0-value', u'bbb'),
             # TODO: This method is run multiple times per edit - cache results?
             if not hasattr(self, 'extras_re'):
-                self.extras_re = re.compile('Package-([a-f0-9-]*)-extras(?:-(.+))?$')
+                self.extras_re = re.compile('([a-zA-Z0-9-]*)-([a-f0-9-]*)-extras(?:-(.+))?$')
                 self.newfield_re = re.compile('newfield(\d+)-(.*)')
             extra_fields = []
             for key, value in self.params.items():
@@ -534,42 +563,266 @@ class ExtrasField(ConfiguredField):
                 if not extras_match:
                     continue
                 key_parts = extras_match.groups()
-                package_id = key_parts[0]
-                if key_parts[1] is None:
+                entity_name = key_parts[0]
+                entity_id = key_parts[1]
+                if key_parts[2] is None:
                     if isinstance(value, dict):
                         # simple dict passed into 'Package-1-extras' e.g. via REST i/f
                         extra_fields.extend(value.items())
-                elif key_parts[1].startswith('newfield'):
-                    newfield_match = self.newfield_re.match(key_parts[1])
+                elif key_parts[2].startswith('newfield'):
+                    newfield_match = self.newfield_re.match(key_parts[2])
                     if not newfield_match:
-                        print 'Warning: did not parse newfield correctly: ', ney_parts
+                        print 'Warning: did not parse newfield correctly: ', key_parts
                         continue
                     new_field_index, key_or_value = newfield_match.groups()
                     if key_or_value == 'key':
                         new_key = value
-                        value_key = 'Package-%s-extras-newfield%s-value' % (package_id, new_field_index)
+                        value_key = '%s-%s-extras-newfield%s-value' % (entity_name, entity_id, new_field_index)
                         new_value = self.params.get(value_key, '')
                         if new_key or new_value:
                             extra_fields.append((new_key, new_value))
                     elif key_or_value == 'value':
                         # if it doesn't have a matching key, add it to extra_fields anyway for
                         # validation to fail
-                        key_key = 'Package-%s-extras-newfield%s-key' % (package_id, new_field_index)
+                        key_key = '%s-%s-extras-newfield%s-key' % (entity_name, entity_id, new_field_index)
                         if not self.params.has_key(key_key):
                             extra_fields.append(('', value))
                     else:
                         print 'Warning: expected key or value for newfield: ', key
-                elif key_parts[1].endswith('-checkbox'):
+                elif key_parts[2].endswith('-checkbox'):
                     continue
                 else:
                     # existing field
-                    key = key_parts[1].decode('utf8')
-                    checkbox_key = 'Package-%s-extras-%s-checkbox' % (package_id, key)
+                    key = key_parts[2].decode('utf8')
+                    checkbox_key = '%s-%s-extras-%s-checkbox' % (entity_name, entity_id, key)
                     delete = self.params.get(checkbox_key, '') == 'on'
                     if not delete:
                         extra_fields.append((key, value))
             return extra_fields
 
+
+class GroupSelectField(ConfiguredField):
+    '''A form field for selecting groups'''
+    
+    def __init__(self, name, allow_empty=True, multiple=True, user_editable_groups=None):
+        super(GroupSelectField, self).__init__(name)
+        self.allow_empty = allow_empty
+        self.multiple = multiple
+        if user_editable_groups == None:
+            raise Exception, "Group select field 'user_editable_groups' is not initialized."
+        self.user_editable_groups = user_editable_groups
+    
+    def get_configured(self):
+        field = self.GroupSelectionField(self.name, self.allow_empty).with_renderer(self.GroupSelectEditRenderer)
+        field.set(multiple=self.multiple)
+        field.user_editable_groups = self.user_editable_groups
+        return field
+
+    class GroupSelectionField(formalchemy.Field):
+        def __init__(self, name, allow_empty):
+            formalchemy.Field.__init__(self, name)
+            self.allow_empty = allow_empty
+        
+        def sync(self):
+            if not self.is_readonly():
+                self._update_groups()
+
+        def _update_groups(self):
+            new_group_ids = self._deserialize() or []
+            
+            # Get groups which have alread been associated.
+            old_groups = self.parent.model.groups
+
+            # Calculate which to append and which to remove.
+            editable_set = set([g.id for g in self.user_editable_groups])
+            old_group_ids = [g.id for g in old_groups]
+            new_set = set(new_group_ids)
+            old_set = set(old_group_ids)
+            append_set = (new_set - old_set).intersection(editable_set)
+            remove_set = (old_set - new_set).intersection(editable_set)
+            
+            # Create package group associations.
+            for id in append_set:
+                group = model.Session.query(model.Group).autoflush(False).get(id)
+                if group:
+                    self.parent.model.groups.append(group)
+
+            # Delete package group associations.
+            for group in self.parent.model.groups:
+                if group.id in remove_set:
+                    self.parent.model.groups.remove(group)
+            
+        def requires_label(self):
+            return False
+        requires_label = property(requires_label)
+
+    class GroupSelectEditRenderer(formalchemy.fields.FieldRenderer):
+        def _get_value(self, **kwargs):
+            return self.field.parent.model.groups
+
+        def _get_user_editable_groups(self):
+            return self.field.user_editable_groups
+       
+        def render(self, **kwargs):
+            # Get groups which are editable by the user.
+            editable_groups = self._get_user_editable_groups()
+
+            # Get groups which are already selected.
+            selected_groups = self._get_value()
+
+            # Make checkboxes HTML from selected groups.
+            checkboxes_html = ''
+            checkbox_action = '<input type="checkbox" name="%(name)s" checked="checked" value="%(id)s" />'
+            checkbox_noaction = '&nbsp;'
+            checkbox_template = '''
+            <dt>
+                %(action)s
+            </dt>
+            <dd>
+                <label for="%(name)s">%(title)s</label><br/>
+            </dd>
+            '''
+            for group in selected_groups:
+                checkbox_context = {
+                    'id': group.id,
+                    'name': self.name + '-' + group.id,
+                    'title': group.title
+                }
+                action = checkbox_noaction
+                if group in editable_groups:
+                    context = {
+                        'id': group.id,
+                        'name': self.name + '-' + group.id
+                    }
+                    action = checkbox_action % context
+                # Make checkbox HTML from a group.
+                checkbox_context = {
+                    'action': action,
+                    'name': self.name + '-' + group.id,
+                    'title': group.title
+                }
+                checkbox_html = checkbox_template % checkbox_context
+                checkboxes_html += checkbox_html
+
+            # Infer addable groups, subtract selected from editable groups.
+            addable_groups = []
+            for group in editable_groups:
+                if group not in selected_groups:
+                    addable_groups.append(group)
+
+            # Construct addable options from addable groups.
+            options = []
+            if len(addable_groups):
+                if self.field.allow_empty or len(selected_groups):
+                    options.append(('', _('(None)')))
+            for group in addable_groups:
+                options.append((group.id, group.title))
+
+            # Make select HTML.
+            if len(options):
+                new_name = self.name + '-new'
+                select_html = h.select(new_name, None, options)
+            else:
+                # Todo: Translation call.
+                select_html = _("Cannot add any groups.")
+
+            # Make the field HTML.
+            field_template = '''  
+        <dl> %(checkboxes)s      
+            <dt>
+                %(label)s
+            </dt>
+            <dd> %(select)s
+            </dd>
+        </dl>
+            '''
+            field_context = {
+                'checkboxes': checkboxes_html,
+                'select': select_html,
+                'label': _("Group"),
+            } 
+            field_html = field_template % field_context
+
+            # Convert to literals.
+            return h.literal(field_html)
+
+        def render_readonly(self, **kwargs):
+            return field_readonly_renderer(self.field.key, self._get_value())
+
+        def _serialized_value(self):
+            name = self.name.encode('utf-8')
+            return [v for k, v in self.params.items() if k.startswith(name)]
+        
+        def deserialize(self):
+            # Return groups which have just been selected by the user.
+            new_group_ids = self._serialized_value()
+            if new_group_ids and isinstance(new_group_ids, list):
+                # Either...
+                if len(new_group_ids) == 1:
+                    # Convert [['id1', 'id2', ...]] into ['id1,' 'id2', ...].
+                    nested_value = new_group_ids[0]
+                    if isinstance(new_group_ids, list):
+                        new_group_ids = nested_value
+                # Or...
+                else:
+                    # Convert [['id1'], ['id2'], ...] into ['id1,' 'id2', ...].
+                    for (i, nested_value) in enumerate(new_group_ids):
+                        if nested_value and isinstance(nested_value, list):
+                            if len(nested_value) > 1:
+                                msg = "Can't derived new group selection from "
+                                msg += "serialized value structured like this:"
+                                msg += " %s" % nested_value
+                                raise Exception, msg
+                            new_group_ids[i] = nested_value[0]
+                # Todo: Decide on the structure of a multiple-group selection.
+            
+            if new_group_ids and isinstance(new_group_ids, basestring):
+                new_group_ids = [new_group_ids]
+
+            return new_group_ids
+
+            
+
+
+class SelectExtraField(TextExtraField):
+    '''A form field for text suggested from from a list of options, that is
+    stored in an "extras" field.'''
+    
+    def __init__(self, name, options, allow_empty=True):
+        self.options = options[:]
+        self.allow_empty = allow_empty
+        # ensure options have key and value, not just a value
+        for i, option in enumerate(self.options):
+            if not isinstance(option, (tuple, list)):
+                self.options[i] = (option, option)
+        super(SelectExtraField, self).__init__(name)
+
+    def get_configured(self):
+        field = self.TextExtraField(self.name, options=self.options)
+        field.allow_empty = self.allow_empty
+        return field.with_renderer(self.SelectRenderer)
+        
+
+    class SelectRenderer(formalchemy.fields.FieldRenderer):
+        def _get_value(self, **kwargs):
+            extras = self.field.parent.model.extras
+            return self.value 
+
+        def render(self, options, **kwargs):
+            selected = self._get_value()
+            if self.field.allow_empty:
+                options = [(_('(None)'), '')] + options
+            
+            html = literal(fa_h.select(self.name, selected, options, **kwargs))
+            return html
+
+        def render_readonly(self, **kwargs):
+            return field_readonly_renderer(self.field.key, self._get_value())
+
+        def _serialized_value(self):
+            return self.params.get(self.name, u'')
+
+
 class SuggestedTextExtraField(TextExtraField):
     '''A form field for text suggested from from a list of options, that is
     stored in an "extras" field.'''
@@ -649,6 +902,37 @@ class CheckboxExtraField(TextExtraField):
         def deserialize(self):
             return u'yes' if self._serialized_value() else u'no'
 
+
+class PackageNameField(ConfiguredField):
+    
+    def get_configured(self):
+        return self.PackageNameField(self.name).with_renderer(self.PackageNameRenderer)
+
+    class PackageNameField(formalchemy.Field):
+        #def sync(self):
+        #    if not self.is_readonly():
+        pass
+        
+    class PackageNameRenderer(formalchemy.fields.FieldRenderer):
+        #def _get_value(self):
+        #    package_id = self.field.parent.model.package_id
+        #    pkg = model.Package.get(package_id)
+        #    return pkg.name
+        pass
+
+
+class UserNameField(ConfiguredField):
+
+    def get_configured(self):
+        return self.UserNameField(self.name).with_renderer(self.UserNameRenderer)
+
+    class UserNameField(formalchemy.Field):
+        pass
+
+    class UserNameRenderer(formalchemy.fields.FieldRenderer):
+        pass
+
+
 def prettify(field_name):
     '''Generates a field label based on the field name.
     Used by the FormBuilder in method set_label_prettifier.
