@@ -3,6 +3,8 @@ import sys
 import logging
 
 import paste.script
+from paste.script.util.logging_config import fileConfig
+import re
 
 class CkanCommand(paste.script.command.Command):
     parser = paste.script.command.Command.standard_parser(verbose=True)
@@ -22,8 +24,12 @@ class CkanCommand(paste.script.command.Command):
             msg = 'No config file supplied'
             raise self.BadCommand(msg)
         self.filename = os.path.abspath(self.options.config)
+        try:
+            fileConfig(self.filename)
+        except Exception: pass
         conf = appconfig('config:' + self.filename)
         load_environment(conf.global_conf, conf.local_conf)
+        
 
     def _setup_app(self):
         cmd = paste.script.appinstall.SetupCommand('setup-app') 
@@ -42,11 +48,7 @@ class ManageDb(CkanCommand):
     db simple-dump-csv {file-path}
     db simple-dump-json {file-path}
     db send-rdf {talis-store} {username} {password}
-    db load {file-path} # load from a file
-    db load-data4nr {file-path.csv}
-    db load-cospread {file-path.csv}
-    db load-esw {file-path.txt} [{host} {api-key}]
-    db load-ons [{file-path.csv}|recent|days={num-days}|all]
+    db load {file-path} # load a dump from a file
     db migrate06
     db migrate09a
     db migrate09b
@@ -92,16 +94,6 @@ class ManageDb(CkanCommand):
             self.dump_rdf(cmd)
         elif cmd == 'send-rdf':
             self.send_rdf(cmd)
-        elif cmd == 'load-data4nr':
-            import ckan.getdata.data4nr as data_getter
-            self.load_data(data_getter.Data4Nr)
-        elif cmd == 'load-cospread':
-            import ckan.getdata.cospread as data_getter
-            self.load_data(data_getter.Data)
-        elif cmd == 'load-esw':
-            self.load_esw(cmd)
-        elif cmd == 'load-ons':
-            self.load_ons()
         elif cmd == 'migrate06':
             import ckan.lib.converter
             dumper = ckan.lib.converter.Dumper()
@@ -143,15 +135,6 @@ class ManageDb(CkanCommand):
         else:
             print 'Unknown command', cmd
 
-    def load_data(self, data_getter, extension='csv'):
-        if len(self.args) < 2:
-            print 'Need %s file path' % extension
-            return
-        load_path = self.args[1] if len(self.args) == 2 else self.args[1:]
-        data = data_getter()
-        load_func = getattr(data, 'load_%s_into_db' % extension)
-        load_func(load_path)
-
     def simple_dump_csv(self, cmd):
         from ckan import model
         if len(self.args) < 2:
@@ -202,157 +185,11 @@ class ManageDb(CkanCommand):
         talis = ckan.lib.talis.Talis()
         return talis.send_rdf(talis_store, username, password)
 
-    def load_esw(self, cmd):
-        if len(self.args) < 2:
-            print 'Need ESW data file path'
-            return
-        load_path = self.args[1]
-        if len(self.args) > 3:
-            server = self.args[2]
-            if server.startswith('http://'):
-                server = server.strip('http://').strip('/')
-            base_location = 'http://%s/api/rest' % server
-            api_key = self.args[3]
-        else:
-            server = api_key = None
-        print 'Loading ESW data\n  Filename: %s\n  Server hostname: %s\n  Api-key: %s' % \
-              (load_path, server, api_key)
-        import ckan.getdata.esw
-        data = ckan.getdata.esw.Esw()
-        if server:
-            import ckanclient
-            ckanclient = ckanclient.CkanClient(base_location=base_location, api_key=api_key)
-            data.load_esw_txt_via_rest(load_path, ckanclient)
-        else:
-            data.load_esw_txt_into_db(load_path)
-
-    def load_ons(self):
-        if len(self.args) < 2:
-            print 'Need xml file path or "all" or "recent" or "days=x".'
-            return
-        arg = self.args[1]
-        if arg == 'recent' or arg == 'all' or arg.startswith('days'):
-            import ckan.getdata.ons_download as ons
-            if len(self.args) < 3:
-                ons_cache_dir = '~/ons_data'
-                print 'Defaulting ONS cache dir: %s' % ons_cache_dir
-            else:
-                ons_cache_dir = self.args[2]
-                print 'ONS cache dir: %s' % ons_cache_dir
-        ons_cache_dir = os.path.expanduser(ons_cache_dir)
-        if not os.path.exists(ons_cache_dir):
-            print 'Creating dir: %s' % ons_cache_dir
-            os.makedirs(ons_cache_dir)
-        if arg == 'recent':
-            new_packages, num_packages_after = ons.import_recent(ons_cache_dir)
-        elif arg.startswith('days='):
-            days = int(arg.split('=')[1])
-            new_packages, num_packages_after = ons.import_recent(ons_cache_dir, days=days)
-        elif arg == 'all':
-            new_packages, num_packages_after = ons.import_all(ons_cache_dir)
-        else:
-            # filename given
-            import ckan.getdata.ons_hub as data_getter
-            self.load_data(data_getter.Data, 'xml')
-
-
-class TestData(CkanCommand):
-    '''Perform simple consistency tests on the db and wui.
-
-    Usage:
-      test-data <wui url>
-    '''
-
-    summary = __doc__.split('\n')[0]
-    usage = __doc__
-    max_args = 1
-    min_args = 1
-
-    def command(self):
-        self._load_config()
-        from ckan import model
-
-        print 'Database check'
-        print '**************'
-
-        num_pkg = model.Session.query(model.Package).count()
-        print '* Number of packages: ', num_pkg
-        assert num_pkg > 0
-
-        num_tag = model.Session.query(model.Tag).count()
-        print '* Number of tags: ', num_tag
-        assert num_tag > 0
-
-        pkg = model.Session.query(model.Package).first()
-        print u'* A package: %s' % pkg.as_dict()
-        expected_attributes = ('name', 'title', 'notes', 'url')
-        for ea in expected_attributes:
-            print '* Checking for attribute ', ea
-            assert ea in pkg.__dict__.keys()
-
-        tag = model.Session.query(model.Tag).first()
-        print '* A tag: ', tag.name
-        expected_attributes = ['name']
-        for ea in expected_attributes:
-            print '* Checking for attribute ', ea
-            assert ea in tag.__dict__.keys()
-
-        print '\nWUI check'
-        print   '========='
-        import paste.fixture
-        self.wui_address = self.args[0]
-        if not self.wui_address.startswith('http://'):
-            self.wui_address = 'http://' + self.wui_address
-        if not self.wui_address.endswith('/'):
-            self.wui_address = self.wui_address + '/'
-
-        import paste.proxy
-        wsgiapp = paste.proxy.make_proxy({}, self.wui_address)
-        self.app = paste.fixture.TestApp(wsgiapp)
-
-        def check_page(path, required_contents, status=200):
-            print "* Checking page '%s%s'" % (self.wui_address, path)
-            res = self.app.get(path, status=status)
-            if type(required_contents) is type(()):
-                for required in required_contents:
-                    print '    ...checking for %r' % required
-                    assert required in res, res
-            else:
-                assert required_contents in res, res
-            return res
-
-        
-        import urllib
-        res = check_page('/', ('Search'))
-        form = res.forms['package-search']
-        form['q'] = pkg.title
-        res = form.submit()
-        print '* Checking search using %r' % pkg.title.encode('utf-8')
-        assert ('package found' in res) or ('packages found' in res), res
-        
-        res = res.click(pkg.title)
-        print '* Checking package page %s' % res.request.url
-        assert pkg.title in res, res
-        for tag in pkg.tags:
-            assert tag.name in res, res
-        
-        if pkg.license:
-            license = pkg.license.as_dict().get('title')
-            assert license in res, res
-
-        tag = pkg.tags[0]
-        res = check_page('/tag/%s' % tag.name, 
-                ('Tag: %s' % str(tag.name), str(pkg.name))
-            )
-
-        res = check_page('/package/new', 'Register a New Data Package')
-        res = check_page('/package/list', 'Data Packages')
 
 class SearchIndexCommand(CkanCommand):
     '''Creates a search index for all packages
 
     Usage:
-      search-index                         - indexes changes as they occur
       search-index rebuild                 - indexes all packages
     '''
 
@@ -363,31 +200,26 @@ class SearchIndexCommand(CkanCommand):
 
     def command(self):
         self._load_config()
-        logging.basicConfig(level=logging.DEBUG)
-        from ckan.lib.search import get_backend, rebuild, SearchIndexWorker
 
         if not self.args:
             # default to run
-            cmd = 'run'
+            cmd = 'rebuild'
         else:
             cmd = self.args[0]
-        if cmd == 'run':
-            backend = get_backend()
-            indexer = SearchIndexWorker(backend)
-            indexer.clear_queue()
-            while True:
-                indexer.run()
+        
         if cmd == 'rebuild':
             rebuild()
         else:
             print 'Command %s not recognized' % cmd
 
+
 class Sysadmin(CkanCommand):
     '''Gives sysadmin rights to a named 
 
     Usage:
       sysadmin list                 - lists sysadmins
       sysadmin create <user-name>   - creates sysadmin user
+      sysadmin remove <user-name>   - removes user fr
     '''
 
     summary = __doc__.split('\n')[0]
@@ -405,7 +237,7 @@ class Sysadmin(CkanCommand):
         elif cmd == 'create':
             self.create()
         elif cmd == 'remove':
-            self.create()
+            self.remove()
         else:
             print 'Command %s not recognized' % cmd
 
@@ -918,7 +750,6 @@ class Notifications(CkanCommand):
 
     Usage:
       notifications monitor                 - runs monitor, printing all notifications
-      notifications replay                  - simulate a change to all packages in the system
     '''
 
     summary = __doc__.split('\n')[0]
@@ -938,23 +769,297 @@ class Notifications(CkanCommand):
         cmd = self.args[0]
         if cmd == 'monitor':
             self.monitor()
-        if cmd == 'replay':
-            self.replay()
         else:
             print 'Command %s not recognized' % cmd
 
-    def replay(self):
-        from ckan.model import Package, PackageResource
-        from ckan.model.notifier import DomainObjectNotificationOperation, DomainObjectNotification
-        for klass in [Package, PackageResource]:
-            for obj in klass.active():
-                if hasattr(obj, 'name'):
-                    print "Simulating change to:", obj.name
-                elif hasattr(obj, 'url'):
-                    print "Simulating change to:", obj.url
-                notification = DomainObjectNotification.create(obj, DomainObjectNotificationOperation.changed)
-                notification.send_synchronously()
-
     def monitor(self):
         from ckan.lib import monitor
         monitor = monitor.Monitor()
+
+class Load(CkanCommand):
+    '''Load data to a remote CKAN instance.
+
+    load bis {filepath.xls} {ckan-api-url} {api-key}
+    load ons {number_of_days_worth} {ckan-api-url} {api-key}
+    '''
+    summary = __doc__.split('\n')[0]
+    usage = __doc__
+    max_args = None
+    min_args = 1
+
+    def command(self):
+        # Avoids vdm logging warning
+        logging.basicConfig(level=logging.ERROR)
+        
+        self._load_config()
+        from ckan import model
+
+        data_type = self.args[0]
+        if data_type == 'bis':
+            if len(self.args) != 4:
+                print 'Error: Wrong number of arguments for data type: %s' % data_type
+                print self.usage
+                sys.exit(1)
+            filepath, api_url, api_key = self.args[1:]
+            assert api_url.startswith('http://') and api_url.endswith('/api'), api_url
+            # import them
+            from ckanext.dgu.bis.bis import BisImporter, BisLoader
+            importer = BisImporter(filepath=filepath)
+            loader = BisLoader
+        elif data_type == 'ons':
+            from ckanext.dgu.ons.importer import OnsImporter
+            from ckanext.dgu.ons.loader import OnsLoader
+            from ckanext.dgu.ons.downloader import OnsData
+            if len(self.args) != 4:
+                print 'Error: Wrong number of arguments for data type: %s' % data_type
+                print self.usage
+                sys.exit(1)
+            days, api_url, api_key = self.args[1:]
+            days = int(days)
+            assert api_url.startswith('http://') and api_url.endswith('/api'), api_url
+            # download data
+            ons_data = OnsData()
+            url, url_name = ons_data._get_url_recent(days=days)
+            data_filepath = ons_data.download(url, url_name, force_download=True)
+            
+            # import them
+            importer = OnsImporter(filepath=data_filepath)
+            loader = OnsLoader
+        else:
+            print 'Error: Data type %r not recognised' % data_type
+            print self.usage
+            sys.exit(1)
+
+        pkg_dicts = [pkg_dict for pkg_dict in importer.pkg_dict()]
+        print '%i packages' % len(pkg_dicts)
+        if pkg_dicts:
+            #raw_input('Press return to load packages')
+            print 'Loading packages...'
+            # load them
+            from ckanclient import CkanClient
+            client = CkanClient(api_key=api_key, base_location=api_url,
+                                is_verbose=False)
+            loader = loader(client)
+            res = loader.load_packages(pkg_dicts)
+            if res['num_errors'] == 0 and res['num_loaded']:
+                print 'SUCCESS'
+            else:
+                print '%i ERRORS' % res['num_errors']
+            print '%i package loaded' % res['num_loaded']
+
+            if res['num_loaded']:
+                #raw_input('Press return to add them to the ukgov group')
+
+                # add them to the group
+                print 'Adding them to the ukgov group...'
+                loader.add_pkgs_to_group(res['pkg_names'], 'ukgov')
+                print '...SUCCESS'
+
+class Harvester(CkanCommand):
+    '''Harvests remotely mastered metadata
+
+    Usage:
+      harvester source {url} [{user-ref} [{publisher-ref}]]     
+        - create new harvest source
+
+      harvester sources                                 
+        - lists harvest sources
+
+      harvester job {source-id} [{user-ref}]
+        - create new harvesting job
+
+      harvester jobs
+        - lists harvesting jobs
+
+      harvester run
+        - runs harvesting jobs
+    '''
+
+    summary = __doc__.split('\n')[0]
+    usage = __doc__
+    max_args = 4
+    min_args = 0
+
+    def command(self):
+        self._load_config()
+        # Clear the 'No handlers could be found for logger "vdm"' warning message.
+        print ""
+        cmd = self.args[0]
+        if cmd == 'source':
+            if len(self.args) >= 2:
+                url = unicode(self.args[1])
+            else:
+                print self.usage
+                print 'Error, source url is not given.'
+                sys.exit(1)
+            if len(self.args) >= 3:
+                user_ref = unicode(self.args[2])
+            else:
+                user_ref = u''
+            if len(self.args) >= 4:
+                publisher_ref = unicode(self.args[3])
+            else:
+                publisher_ref = u''
+            self.register_harvest_source(url, user_ref, publisher_ref)
+        elif cmd == 'sources':
+            self.list_harvest_sources()
+        elif cmd == 'job':
+            if len(self.args) >= 2:
+                source_id = unicode(self.args[1])
+            else:
+                print self.usage
+                print 'Error, job source is not given.'
+                sys.exit(1)
+            if len(self.args) >= 3:
+                user_ref = unicode(self.args[2])
+            else:
+                user_ref = u''
+            self.register_harvesting_job(source_id, user_ref)
+        elif cmd == 'jobs':
+            self.list_harvesting_jobs()
+        elif cmd == 'run':
+            self.run_harvester()
+        else:
+            print 'Command %s not recognized' % cmd
+
+    def _load_config(self):
+        super(Harvester, self)._load_config()
+        import logging
+        logging.basicConfig()
+        logger_vdm = logging.getLogger('vdm')
+        logger_vdm.setLevel(logging.ERROR)
+
+    def run_harvester(self, *args, **kwds):
+        from pylons.i18n.translation import _get_translator
+        import pylons
+        pylons.translator._push_object(_get_translator(pylons.config.get('lang')))
+
+        from ckan.model import HarvestingJob
+        jobs = HarvestingJob.filter(status=u"New").all()
+        jobs_len = len(jobs)
+        jobs_count = 0
+        if jobs_len:
+            print "Running %s harvesting jobs..." % jobs_len
+        else:
+            print "There are no new harvesting jobs."
+        print ""
+        for job in jobs:
+            jobs_count += 1
+            print "Running job %s/%s: %s" % (jobs_count, jobs_len, job.id)
+            job.harvest_documents()
+            #self.print_harvesting_job(job)
+            print ""
+            job = HarvestingJob.get(job.id)
+            self.print_harvesting_job(job)
+
+
+    def register_harvesting_job(self, source_id, user_ref):
+        from ckan.model import HarvestSource
+        from ckan.model import HarvestingJob
+        if re.match('(http|file)://', source_id):
+            source_url = unicode(source_id)
+            source_id = None
+            sources = HarvestSource.filter(url=source_url).all()
+            if sources:
+                source = sources[0]
+            else:
+                source = self.create_harvest_source(url=source_url, user_ref=user_ref, publisher_ref=u'')
+        else:
+            source = HarvestSource.get(source_id)
+
+        job = HarvestingJob.create_save(source=source, user_ref=user_ref, status=u"New")
+        print "Created new harvesting job:"
+        self.print_harvesting_job(job)
+        status = u"New"
+        jobs = HarvestingJob.filter(status=status).all()
+        self.print_there_are("harvesting job", jobs, condition=status)
+
+    def register_harvest_source(self, url, user_ref, publisher_ref):
+        from ckan.model import HarvestSource
+        existing = self.get_harvest_sources(url=url)
+        if existing:
+            print "Error, there is already a harvesting source for that URL"
+            self.print_harvest_sources(existing)
+            sys.exit(1)
+        else:
+            source = self.create_harvest_source(url=url, user_ref=user_ref, publisher_ref=publisher_ref)
+            print "Created new harvest source:"
+            self.print_harvest_source(source)
+            sources = self.get_harvest_sources()
+            self.print_there_are("harvest source", sources)
+
+    def list_harvest_sources(self):
+        sources = self.get_harvest_sources()
+        self.print_harvest_sources(sources)
+        self.print_there_are(what="harvest source", sequence=sources)
+       
+    def list_harvesting_jobs(self):
+        jobs = self.get_harvesting_jobs()
+        self.print_harvesting_jobs(jobs)
+        self.print_there_are(what="harvesting job", sequence=jobs)
+
+    def get_harvest_sources(self, **kwds):
+        from ckan.model import HarvestSource
+        return HarvestSource.filter(**kwds).all()
+
+    def get_harvesting_jobs(self, **kwds):
+        from ckan.model import HarvestingJob
+        return HarvestingJob.filter(**kwds).all()
+
+    def create_harvest_source(self, **kwds):
+        from ckan.model import HarvestSource
+        return HarvestSource.create_save(**kwds)
+
+    def create_harvesting_job(self, **kwds):
+        from ckan.model import HarvestingJob
+        return HarvestingJob.create_save(**kwds)
+
+    def print_harvest_sources(self, sources):
+        if sources:
+            print ""
+        for source in sources:
+            self.print_harvest_source(source)
+
+    def print_harvest_source(self, source):
+        print "Source id: %s" % source.id
+        print "      url: %s" % source.url
+        print "     user: %s" % source.user_ref
+        print "publisher: %s" % source.publisher_ref
+        print "     docs: %s" % len(source.documents)
+        print ""
+
+    def print_harvesting_jobs(self, jobs):
+        if jobs:
+            print ""
+        for job in jobs:
+            self.print_harvesting_job(job)
+
+    def print_harvesting_job(self, job):
+        print "Job id: %s" % job.id
+        if job.user_ref:
+            print "  user: %s" % job.user_ref
+        print "status: %s" % job.status
+        print "source: %s" % job.source.id
+        print "   url: %s" % job.source.url
+        #print "report: %s" % job.report
+        if job.report and job.report['packages']:
+            for package_id in job.report['packages']:
+                print "   doc: %s" % package_id
+        if job.report and job.report['errors']:
+            for msg in job.report['errors']:
+                print " error: %s" % msg
+        print ""
+
+    def print_there_are(self, what, sequence, condition=""):
+        is_singular = self.is_singular(sequence)
+        print "There %s %s %s%s%s" % (
+            is_singular and "is" or "are",
+            len(sequence),
+            condition and ("%s " % condition.lower()) or "",
+            what,
+            not is_singular and "s" or "",
+        )
+
+    def is_singular(self, sequence):
+        return len(sequence) == 1
+    
