@@ -8,6 +8,7 @@ class CreateTestData(cli.CkanCommand):
     create-test-data search  - realistic data to test search
     create-test-data gov     - government style data
     create-test-data family  - package relationships data
+    create-test-data user    - create a user 'tester' with api key 'tester'
     '''
     summary = __doc__.split('\n')[0]
     usage = __doc__
@@ -36,6 +37,8 @@ class CreateTestData(cli.CkanCommand):
             print 'Creating %s test data' % cmd
         if cmd == 'basic':
             self.create_basic_test_data()
+        elif cmd == 'user':
+            self.create_user()
         elif cmd == 'search':
             self.create_search_test_data()
         elif cmd == 'gov':
@@ -67,10 +70,27 @@ class CreateTestData(cli.CkanCommand):
                               extra_user_names=extra_users)
 
     @classmethod
-    def create_arbitrary(self, package_dicts,
-                         relationships=[], extra_user_names=[],
-                         extra_group_names=[],
-                         commit_changesets=False):
+    def create_user(self):
+        import ckan.model as model
+        tester = model.User.by_name(u'tester')
+        if tester is None:
+            tester = model.User(name=u'tester', apikey=u'tester')
+            model.Session.add(tester)
+            model.Session.commit()
+        model.Session.remove()
+        self.user_names = [u'tester']
+
+    @classmethod
+    def create_arbitrary(self, package_dicts, relationships=[],
+            extra_user_names=[], extra_group_names=[], 
+            commit_changesets=False, admins=[]):
+        '''Creates packages and a few extra objects as well at the
+        same time if required.
+        @param package_dicts - a list of dictionaries with the package
+                               properties
+        @param extra_group_names - a list of group names to create. No
+                               properties get set though.
+        '''
         assert isinstance(relationships, (list, tuple))
         assert isinstance(extra_user_names, (list, tuple))
         assert isinstance(extra_group_names, (list, tuple))
@@ -78,12 +98,13 @@ class CreateTestData(cli.CkanCommand):
         model.Session.remove()
         new_user_names = extra_user_names
         new_group_names = set()
-
+        
+        rev = model.repo.new_revision() 
+        rev.author = self.author
+        rev.message = u'Creating test packages.'
+        
         admins_list = [] # list of (package_name, admin_names)
         if package_dicts:
-            rev = model.repo.new_revision() 
-            rev.author = self.author
-            rev.message = u'Creating test packages.'
             if isinstance(package_dicts, dict):
                 package_dicts = [package_dicts]
             for item in package_dicts:
@@ -134,9 +155,9 @@ class CreateTestData(cli.CkanCommand):
                         else:
                             raise NotImplementedError
                         for group_name in group_names:
-                            group = model.Group.by_name(group_name)
+                            group = model.Group.by_name(unicode(group_name))
                             if not group:
-                                group = model.Group(name=group_name)
+                                group = model.Group(name=unicode(group_name))
                                 model.Session.add(group)
                                 new_group_names.add(group_name)
                             pkg.groups.append(group)
@@ -147,6 +168,7 @@ class CreateTestData(cli.CkanCommand):
                     elif attr == 'extras':
                         pkg.extras = val
                     elif attr == 'admins':
+                        # Todo: Use admins parameter to pass in admins (three tests).
                         assert isinstance(val, list)
                         admins_list.append((item['name'], val))
                         for user_name in val:
@@ -155,11 +177,12 @@ class CreateTestData(cli.CkanCommand):
                     else:
                         raise NotImplementedError(attr)
                 self.pkg_names.append(item['name'])
-                model.setup_default_user_roles(pkg)
+                model.setup_default_user_roles(pkg, admins=admins)
             model.repo.commit_and_remove()
 
         needs_commit = False
-
+        
+        rev = model.repo.new_revision() 
         for group_name in extra_group_names:
             group = model.Group(name=unicode(group_name))
             model.Session.add(group)
@@ -214,10 +237,38 @@ class CreateTestData(cli.CkanCommand):
             from ckan.model.changeset import ChangesetRegister
             changeset_ids = ChangesetRegister().commit()
 
+    @classmethod
+    def create_groups(self, group_dicts, admin_user_name):
+        '''A more featured interface for creating groups.
+        All group fields can be filled, packages added and they can
+        have an admin user.'''
+        import ckan.model as model
+        rev = model.repo.new_revision()
+        # same name as user we create below
+        rev.author = self.author
+        admin_user = model.User.by_name(admin_user_name)
+        assert isinstance(group_dicts, (list, tuple))
+        for group_dict in group_dicts:
+            group = model.Group(name=unicode(group_dict['name']))
+            for key in ('title', 'description'):
+                if group_dict.has_key(key):
+                    setattr(group, key, group_dict[key])
+            pkg_names = group_dict.get('packages', [])
+            assert isinstance(pkg_names, (list, tuple))
+            for pkg_name in pkg_names:
+                pkg = model.Package.by_name(unicode(pkg_name))
+                assert pkg, pkg_name
+                pkg.groups.append(group)
+            model.Session.add(group)
+            model.setup_default_user_roles(group, [admin_user])
+            self.group_names.add(group_dict['name'])
+        model.repo.commit_and_remove()
+
     @classmethod
     def create(self, commit_changesets=False):
         import ckan.model as model
         model.Session.remove()
+        self.create_user()
         rev = model.repo.new_revision()
         # same name as user we create below
         rev.author = self.author
@@ -280,10 +331,6 @@ left arrow <
         pkg2.title = u'A Wonderful Story'
         pkg1.extras = {u'genre':'romantic novel',
                        u'original media':'book'}
-        # api key
-        tester = model.User(name=u'tester', apikey=u'tester')
-        model.Session.add(tester)
-        self.user_names = [u'tester']
         # group
         david = model.Group(name=u'david',
                              title=u'Dave\'s books',
@@ -331,6 +378,8 @@ left arrow <
         '''If you create a domain object manually in your test then you
         can name it here (flag it up) and it will be deleted when you next
         call CreateTestData.delete().'''
+        if isinstance(pkg_names, basestring):
+            pkg_names = [pkg_names]
         self.pkg_names.extend(pkg_names)
         self.tag_names.extend(tag_names)
         self.group_names = self.group_names.union(set(group_names))
@@ -354,21 +403,35 @@ left arrow <
             tag = model.Tag.by_name(unicode(tag_name))
             if tag:
                 tag.purge()
-        revs = model.Session.query(model.Revision).filter_by(author=self.author)
-        for rev in revs:
-            for pkg in rev.packages:
-                pkg.purge()
-            model.Session.delete(rev)
         for group_name in self.group_names:
             group = model.Group.by_name(unicode(group_name))
             if group:
                 model.Session.delete(group)
+        revs = model.Session.query(model.Revision).filter_by(author=self.author)
+        for rev in revs:
+            for pkg in rev.packages:
+                pkg.purge()
+            for grp in rev.groups:
+                grp.purge()
+            model.Session.delete(rev)
         for user_name in self.user_names:
             user = model.User.by_name(unicode(user_name))
             if user:
                 user.purge()
         model.Session.commit()
         model.Session.remove()
+        self.reset()
+
+    @classmethod
+    def reset(cls):
+        cls.pkg_names = []
+        cls.group_names = set()
+        cls.tag_names = []
+        cls.user_names = []
+
+    @classmethod
+    def get_all_data(cls):
+        return cls.pkg_names + list(cls.group_names) + cls.tag_names + cls.user_names
 
 
 search_items = [{'name':'gils',
@@ -518,7 +581,7 @@ gov_items = [
     {'name':'weekly-fuel-prices',
      'title':'Weekly fuel prices',
      'notes':'Latest price as at start of week of unleaded petrol and diesel.',
-     'resources':[{'url':'', 'format':'XLS', 'description':''}],
+     'resources':[{'url':'http://www.decc.gov.uk/en/content/cms/statistics/prices.xls', 'format':'XLS', 'description':''}],
      'url':'http://www.decc.gov.uk/en/content/cms/statistics/source/prices/prices.aspx',
      'author':'DECC Energy Statistics Team',
      'author_email':'energy.stats@decc.gsi.gov.uk',
