@@ -11,11 +11,19 @@ from webhelpers.html.tags import *
 from webhelpers.markdown import markdown
 from webhelpers import paginate
 from webhelpers.text import truncate
+from pylons.decorators.cache import beaker_cache
 from routes import url_for, redirect_to
 from alphabet_paginate import AlphaPage
+from lxml.html import fromstring
+
+try:
+    from collections import OrderedDict # from python 2.7
+except ImportError:
+    from sqlalchemy.util import OrderedDict
+
 try:
     import json
-except Exception:
+except ImportError:
     import simplejson as json
 
 # FIXME: shouldn't have to pass the c object in to this.
@@ -34,11 +42,39 @@ def subnav_link(c, text, action, **kwargs):
         class_=('active' if c.action == action else '')
     )
 
+def am_authorized(c, action, domain_object=None):
+    from ckan.authz import Authorizer
+    if domain_object is None:
+        from ckan import model
+        domain_object = model.System()
+    return Authorizer.am_authorized(c, action, domain_object)
+
+def linked_user(user):
+    from ckan import model
+    from urllib import quote
+    if user in [model.PSEUDO_USER__LOGGED_IN, model.PSEUDO_USER__VISITOR]:
+        return user
+    if not isinstance(user, model.User):
+        user = model.User.get(unicode(user))
+    if user:
+        _name = user.name if model.User.VALID_NAME.match(user.name) else user.id
+        _icon = icon("user") + " "
+        return _icon + link_to(user.display_name, 
+                       url_for(controller='user', action='read', id=_name))
+    return user
+
+def markdown_extract(text):
+    if (text is None) or (text == ''):
+        return ''
+    html = fromstring(markdown(text))
+    plain = html.xpath("string()")
+    return unicode(truncate(plain, length=270, indicator='...', whole_word=True))
+
 def icon_url(name):
     return '/images/icons/%s.png' % name
 
 def icon(name, alt=None):
-    return literal('<img src="%s" height="16px" width="16px" alt="%s" />' % (icon_url(name), alt))
+    return literal('<img src="%s" height="16px" width="16px" alt="%s" /> ' % (icon_url(name), alt))
 
 class Page(paginate.Page):
     
@@ -46,8 +82,8 @@ class Page(paginate.Page):
     # our custom layout set as default.
     def pager(self, *args, **kwargs):
         kwargs.update(
-            format="<div class='pager'>$link_previous ~2~ $link_next</div>",
-            symbol_previous='« Prev', symbol_next='Next »'
+            format=u"<div class='pager'>$link_previous ~2~ $link_next</div>",
+            symbol_previous=u'« Prev', symbol_next=u'Next »'
         )
         return super(Page, self).pager(*args, **kwargs)
 
