@@ -17,13 +17,13 @@ class PackageSaver(object):
     @classmethod
     def render_preview(cls, fs, original_name, record_id,
                        log_message=None,
-                       author=None):
+                       author=None, client=None):
         '''Renders a package on the basis of a fieldset - perfect for
         preview of form data.
         Note that the actual calling of render('package/read') is left
         to the caller.'''
         pkg = cls._preview_pkg(fs, original_name, record_id,
-                               log_message, author)
+                               log_message, author, client=client)
         cls.render_package(pkg)
 
     # TODO: rename to something more correct like prepare_for_render
@@ -42,12 +42,10 @@ class PackageSaver(object):
         c.pkg_author_link = cls._person_email_link(c.pkg.author, c.pkg.author_email, "Author")
         c.pkg_maintainer_link = cls._person_email_link(c.pkg.maintainer, c.pkg.maintainer_email, "Maintainer")
         c.package_relationships = pkg.get_relationships_printable()
-        # c.auth_for_change_state and c.auth_for_edit may also used
-        # return render('package/read')
 
     @classmethod
     def _preview_pkg(cls, fs, original_name, pkg_id,
-                     log_message=None, author=None):
+                     log_message=None, author=None, client=None):
         '''Previews the POST data (associated with a package edit)
         @input c.error
         @input fs      FieldSet with the param data bound to it
@@ -59,7 +57,7 @@ class PackageSaver(object):
         '''
         try:
             out = cls._update(fs, original_name, pkg_id, log_message,
-                              author, commit=False)
+                              author, commit=False, client=client)
             # While pkg is still in the session, touch the relations so they
             # lazy load, for use later.
             fs.model.license
@@ -74,7 +72,7 @@ class PackageSaver(object):
         return out
 
     @classmethod
-    def commit_pkg(cls, fs, original_name, pkg_id, log_message, author):
+    def commit_pkg(cls, fs, original_name, pkg_id, log_message, author, client=None):
         '''Writes the POST data (associated with a package edit) to the
         database
         @input c.error
@@ -82,15 +80,20 @@ class PackageSaver(object):
         @input original_name Name of the package before this edit
         @input pkg_id Package id
         '''
-        cls._update(fs, original_name, pkg_id, log_message, author, commit=True)
+        cls._update(fs, original_name, pkg_id, log_message, author, commit=True, client=client)
 
     @classmethod
-    def _update(cls, fs, original_name, pkg_id, log_message, author, commit=True):
+    def _update(cls, fs, original_name, pkg_id, log_message, author, commit=True, client=None):
+        # Todo: Remove original_name and pkg_id, since they aren't used.
+        # Todo: Consolidate log message field (and validation).
+        # Todo: Separate out the preview line of execution, it's confusing.
         rev = None
         # validation
-        c.errors = cls._revision_validation(log_message)
-        fs_validation = fs.validate() #errors stored in fs.errors
-        validates = not (c.errors or fs.errors)
+        errors = cls._revision_validation(log_message)
+        if client:
+            client.errors = errors
+        fs.validate()
+        validates = not (errors or fs.errors)
 
         # sync
         try:
@@ -135,6 +138,20 @@ class PackageSaver(object):
     @classmethod
     def _person_email_link(cls, name, email, reference):
         if email:
-            return h.mail_to(email_address=email, name=name or email, encode='javascript')
+            return h.mail_to(email_address=email, name=name or email, encode='hex')
         else:
             return name or reference + " not given"
+
+
+class WritePackageFromBoundFieldset(object):
+
+    def __init__(self, fieldset, log_message='', author='', client=None):
+        self.fieldset = fieldset
+        self.log_message = log_message
+        self.author = author
+        self.client = None
+        self.write_package()
+
+    def write_package(self):
+        PackageSaver().commit_pkg(self.fieldset, None, None, self.log_message, self.author, self.client)
+
