@@ -4,9 +4,9 @@ from pylons import config
 from sqlalchemy import *
 
 from ckan import model
+from ckan.lib.cache import cache_enabled
 
-ENABLE_CACHING = bool(config.get('enable_caching', ''))
-if ENABLE_CACHING:
+if cache_enabled:
     from pylons import cache
     our_cache = cache.get_cache('stats', type='dbm')
 DATE_FORMAT = '%Y-%m-%d'
@@ -85,6 +85,7 @@ class Stats(object):
         user_object_role = table('user_object_role')
         s = select([user_object_role.c.user_id, func.count(user_object_role.c.role)], from_obj=[user_object_role.join(package_role)]).\
             where(user_object_role.c.role==model.authz.Role.ADMIN).\
+            where(user_object_role.c.user_id!=None).\
             group_by(user_object_role.c.user_id).\
             order_by(func.count(user_object_role.c.role).desc()).\
             limit(limit)
@@ -166,7 +167,7 @@ class RevisionStats(object):
             for pkg_id, created_datetime in res:
                 res_pickleable.append((pkg_id, created_datetime.toordinal()))
             return res_pickleable
-        if ENABLE_CACHING:
+        if cache_enabled:
             week_commences = self.get_date_week_started(datetime.date.today())
             key = 'all_new_packages_%s' + week_commences.strftime(DATE_FORMAT)
             new_packages = our_cache.get_value(key=key,
@@ -216,7 +217,7 @@ class RevisionStats(object):
                 week_ends = week_commences + datetime.timedelta(days=7)
                 weekly_pkg_ids.append(build_weekly_stats(week_commences, []))
             return weekly_pkg_ids
-        if ENABLE_CACHING:
+        if cache_enabled:
             week_commences = self.get_date_week_started(datetime.date.today())
             key = '%s_by_week_%s' % (self._object_type, week_commences.strftime(DATE_FORMAT))
             objects_by_week_ = our_cache.get_value(key=key,
