@@ -1,18 +1,21 @@
 import meta
+from domain_object import DomainObjectOperation
 from core import *
 from package import *
 from tag import *
 from package_mapping import *
 from user import user_table, User
+from authorization_group import * 
 from group import *
+from group_extra import *
 from search_index import *
 from authz import *
-from extras import *
+from package_extra import *
 from resource import *
 from rating import *
-from package_relationship import PackageRelationship
+from package_relationship import *
 from changeset import Changeset, Change, Changemask
-from notifier import *
+from harvesting import HarvestSource, HarvestingJob, HarvestedDocument
 
 import ckan.migration
 
@@ -44,13 +47,7 @@ class Repository(vdm.sqlalchemy.Repository):
             logged_in = User(name=PSEUDO_USER__LOGGED_IN)
             Session.add(visitor)
             Session.add(logged_in)
-            # setup all role-actions
-            # context is blank as not currently used
-            # Note that Role.ADMIN can already do anything - hardcoded in.
-            for role, action in default_role_actions:
-                ra = RoleAction(role=role, context=u'',
-                        action=action,)
-                Session.add(ra)
+        validate_authorization_setup()
         if Session.query(Revision).count() == 0:
             rev = Revision()
             rev.author = 'system'
@@ -66,6 +63,7 @@ class Repository(vdm.sqlalchemy.Repository):
         # doing paster db clean && paster db upgrade !
         # self.upgrade_db()
         self.setup_migration_version_control(self.latest_migration_version())
+        self.create_indexes()
 
     def latest_migration_version(self):
         import migrate.versioning.api as mig
@@ -81,7 +79,19 @@ class Repository(vdm.sqlalchemy.Repository):
                     self.migrate_repository, version)
         except migrate.versioning.exceptions.DatabaseAlreadyControlledError:
             pass
-
+    
+    def create_indexes(self):
+        import os
+        from migrate.versioning.script import SqlScript
+        from sqlalchemy.exceptions import ProgrammingError
+        try:
+            path = os.path.join(self.migrate_repository, 'versions', '021_postgres_upgrade.sql')
+            script = SqlScript(path) 
+            script.run(meta.engine, step=None)
+        except ProgrammingError, e:
+            if not 'already exists' in repr(e):
+                raise
+    
     def upgrade_db(self, version=None):
         '''Upgrade db using sqlalchemy migrations.
 
@@ -89,13 +99,13 @@ class Repository(vdm.sqlalchemy.Repository):
         '''
         import migrate.versioning.api as mig
         self.setup_migration_version_control()
-        mig.upgrade(self.metadata.bind.url, self.migrate_repository,
-                version=version)
+        mig.upgrade(self.metadata.bind.url, self.migrate_repository, version=version)
+        validate_authorization_setup()
 
 
 
 repo = Repository(metadata, Session,
-        versioned_objects=[Package, PackageTag, PackageResource, PackageExtra]
+        versioned_objects=[Package, PackageTag, PackageResource, PackageExtra, PackageGroup, Group]
         )
 
 
@@ -107,9 +117,21 @@ def _get_packages(self):
         pkgs.add(pkg_rev.continuity)
     for non_pkg_rev_list in changes.values():
         for non_pkg_rev in non_pkg_rev_list:
-            pkgs.add(non_pkg_rev.continuity.package)
+            if hasattr(non_pkg_rev.continuity, 'package'):
+                pkgs.add(non_pkg_rev.continuity.package)
     return list(pkgs)
 
+def _get_groups(self):
+    changes = repo.list_changes(self)
+    groups = set()
+    for group_rev in changes.pop(Group):
+        groups.add(group_rev.continuity)
+    for non_group_rev_list in changes.values():
+        for non_group_rev in non_group_rev_list:
+            if hasattr(non_group_rev.continuity, 'group'):
+                groups.add(non_group_rev.continuity.group)
+    return list(groups)
+
 # could set this up directly on the mapper?
 def _get_revision_user(self):
     username = unicode(self.author)
@@ -117,6 +139,7 @@ def _get_revision_user(self):
     return user
 
 Revision.packages = property(_get_packages)
+Revision.groups = property(_get_groups)
 Revision.user = property(_get_revision_user)
 
 def strptimestamp(s):
