@@ -7,8 +7,8 @@ from domain_object import DomainObject
 revision_table = vdm.sqlalchemy.make_revision_table(metadata)
 
 class System(DomainObject):
-    def __str__(self):
-        return 'System'
+    def __unicode__(self):
+        return u'<%s>' % self.__class__.__name__
     def purge(self):
         pass
 
