@@ -1,5 +1,8 @@
 import datetime
+from time import mktime, gmtime
 
+from sqlalchemy.sql import select, and_, union, expression
+from sqlalchemy.orm import eagerload_all
 from pylons import config
 from meta import *
 import vdm.sqlalchemy
@@ -9,15 +12,18 @@ from core import *
 from license import License, LicenseRegister
 from domain_object import DomainObject
 
-__all__ = ['Package', 'package_table', 'package_revision_table']
+__all__ = ['Package', 'package_table', 'package_revision_table',
+           'PACKAGE_NAME_MAX_LENGTH', 'PACKAGE_VERSION_MAX_LENGTH']
 
+PACKAGE_NAME_MAX_LENGTH = 100
+PACKAGE_VERSION_MAX_LENGTH = 100
 ## Our Domain Object Tables
-
 package_table = Table('package', metadata,
         Column('id', types.UnicodeText, primary_key=True, default=make_uuid),
-        Column('name', types.Unicode(100), unique=True, nullable=False),
+        Column('name', types.Unicode(PACKAGE_NAME_MAX_LENGTH),
+               unique=True, nullable=False),
         Column('title', types.UnicodeText),
-        Column('version', types.Unicode(100)),
+        Column('version', types.Unicode(PACKAGE_VERSION_MAX_LENGTH)),
         Column('url', types.UnicodeText),
         Column('author', types.UnicodeText),
         Column('author_email', types.UnicodeText),
@@ -50,7 +56,10 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
     @classmethod
     def get(cls, reference):
         '''Returns a package object referenced by its id or name.'''
-        pkg = Session.query(cls).get(reference)
+        query = Session.query(cls).filter(cls.id==reference)
+        query = query.options(eagerload_all('package_tags.tag'))
+        query = query.options(eagerload_all('package_resources_all'))
+        pkg = query.first()
         if pkg == None:
             pkg = cls.by_name(reference)            
         return pkg
@@ -133,8 +142,12 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         # Set 'license' in _dict to cater for old clients.
         # Todo: Remove from Version 2?
         _dict['license'] = self.license.title if self.license else _dict.get('license_id', '')
-        _dict['tags'] = [tag.name for tag in self.tags]
-        _dict['groups'] = [getattr(group, ref_group_by) for group in self.groups]
+        tags = [tag.name for tag in self.tags]
+        tags.sort() # so it is determinable
+        _dict['tags'] = tags
+        groups = [getattr(group, ref_group_by) for group in self.groups]
+        groups.sort()
+        _dict['groups'] = groups
         _dict['extras'] = dict([(key, value) for key, value in self.extras.items()])
         _dict['ratings_average'] = self.get_average_rating()
         _dict['ratings_count'] = len(self.ratings)
@@ -144,12 +157,17 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         if site_url:
             _dict['ckan_url'] = '%s/package/%s' % (site_url, self.name)
         _dict['relationships'] = [rel.as_dict(self, ref_package_by=ref_package_by) for rel in self.get_relationships()]
+        _dict['metadata_modified'] = self.metadata_modified.isoformat() \
+            if self.metadata_modified else None
+        _dict['metadata_created'] = self.metadata_created.isoformat() \
+            if self.metadata_created else None
         return _dict
 
     def add_relationship(self, type_, related_package, comment=u''):
         '''Creates a new relationship between this package and a
         related_package. It leaves the caller to commit the change.'''
         import package_relationship
+        from ckan import model
         if type_ in package_relationship.PackageRelationship.get_forward_types():
             subject = self
             object_ = related_package
@@ -160,12 +178,21 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
             object_ = self
         else:
             raise NotImplementedError, 'Package relationship type: %r' % type_
-            
-        rel = package_relationship.PackageRelationship(
-            subject=subject,
-            object=object_,
-            type=type_,
-            comment=comment)
+
+        rels = self.get_relationships(with_package=related_package,
+                                      type=type_, active=False, direction="forward")
+        if rels:
+            rel = rels[0]
+            if comment:
+                rel.comment=comment
+            if rel.state == model.State.DELETED:
+                rel.undelete()
+        else:
+            rel = package_relationship.PackageRelationship(
+                subject=subject,
+                object=object_,
+                type=type_,
+                comment=comment)
         Session.add(rel)
         return rel
 
@@ -216,28 +243,32 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
         for rel in self.get_relationships():
             if rel.subject == self:
                 type_printable = PackageRelationship.make_type_printable(rel.type)
-                rel_list.append((rel.object, type_printable))
+                rel_list.append((rel.object, type_printable, rel.comment))
             else:
                 type_printable = PackageRelationship.make_type_printable(\
                     PackageRelationship.forward_to_reverse_type(
                         rel.type)
                     )
-                rel_list.append((rel.subject, type_printable))
+                rel_list.append((rel.subject, type_printable, rel.comment))
         # sibling types
         # e.g. 'gary' is a child of 'mum', looking for 'bert' is a child of 'mum'
         # i.e. for each 'child_of' type relationship ...
         for rel_as_subject in self.get_relationships(direction='forward'):
+            if rel_as_subject.state != State.ACTIVE:
+                continue
             # ... parent is the object
             parent_pkg = rel_as_subject.object
             # Now look for the parent's other relationships as object ...
             for parent_rel_as_object in parent_pkg.get_relationships(direction='reverse'):
+                if parent_rel_as_object.state != State.ACTIVE:
+                    continue
                 # and check children
                 child_pkg = parent_rel_as_object.subject
                 if child_pkg != self and \
                        parent_rel_as_object.type == rel_as_subject.type:
                     type_printable = PackageRelationship.inferred_types_printable['sibling']
-                    rel_list.append((child_pkg, type_printable))
-        return rel_list
+                    rel_list.append((child_pkg, type_printable, None))
+        return sorted(rel_list)
     #
     ## Licenses are currently integrated into the domain model here.   
  
@@ -299,10 +330,16 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                  cmp(rev_tuple2[0].timestamp, rev_tuple1[0].timestamp)
         return sorted(result_list, cmp=ourcmp)
 
+    @property
+    def latest_related_revision(self):
+        '''Returns the latest revision for the package and its related
+        objects.'''
+        return self.all_related_revisions[0][0]
+        
     def diff(self, to_revision=None, from_revision=None):
         '''Overrides the diff in vdm, so that related obj revisions are
         diffed as well as PackageRevisions'''
-        import extras, resource
+        import package_extra, resource
         results = {} # field_name:diffs
         results.update(super(Package, self).diff(to_revision, from_revision))
         # Iterate over PackageTag, PackageExtra, PackageResources etc.
@@ -344,11 +381,82 @@ class Package(vdm.sqlalchemy.RevisionedObjectMixin,
                         results[key] = value_diff
         return results
 
+    @staticmethod
+    def last_modified(*av):
+        """
+        Return most recent timestamp for a package revision, with optionally
+        extra where clause.
+        """
+        from ckan import model
+        where = []
+        for arg in av:
+            if isinstance(arg, expression.ClauseElement) or isinstance(arg, basestring):
+                where.append(arg)
+        where_clauses = [
+            and_(model.package_table.c.revision_id == model.revision_table.c.id, *where),
+            and_(model.package_extra_table.c.package_id == model.package_table.c.id,
+                 model.package_extra_table.c.revision_id == model.revision_table.c.id, *where),
+            and_(model.package_relationship_table.c.subject_package_id == model.package_table.c.id,
+                 model.package_relationship_table.c.revision_id == model.revision_table.c.id, *where),
+            and_(model.package_relationship_table.c.object_package_id == model.package_table.c.id,
+                 model.package_relationship_table.c.revision_id == model.revision_table.c.id, *where),
+            and_(model.package_resource_table.c.package_id == model.package_table.c.id,
+                 model.package_resource_table.c.revision_id == model.revision_table.c.id, *where),
+            and_(model.package_tag_table.c.package_id == model.package_table.c.id,
+                 model.package_tag_table.c.revision_id == model.revision_table.c.id, *where)
+            ]
+        query = union(*[select([model.revision_table.c.timestamp], x) for x in where_clauses]
+                      ).order_by("timestamp DESC").limit(1)
+        conn = model.meta.engine.connect()
+        result = conn.execute(query).fetchone()
+        if result:
+            timestamp = result[0].utctimetuple()
+            usecs = float("0.%s" % result[0].microsecond)
+        else:
+            timestamp, usecs = gmtime(), 0
+        return mktime(timestamp) + usecs
+
+    @property
+    def metadata_modified(self):
+        import ckan.model as model
+        epochtime = self.last_modified(model.package_table.c.id==self.id)
+        return datetime.datetime.fromtimestamp(epochtime)
+    
+    @property
+    def metadata_created(self):
+        import ckan.model as model
+        q = model.Session.query(model.Revision).select_from(
+            model.revision_table.join(
+                model.package_revision_table,
+                and_(model.revision_table.c.id==model.package_revision_table.c.revision_id))
+            ).order_by(model.Revision.timestamp.asc())
+        ts = q.first()
+        if ts is not None:
+            return ts.timestamp
+
+    @staticmethod
+    def get_fields(core_only=False, fields_to_ignore=None):
+        '''Returns a list of the properties of a package.
+        @param core_only - limit it to fields actually in the package table and
+                           not those on related objects, such as tags & extras.
+        @param fields_to_ignore - a list of names of fields to not return if
+                           present.
+        '''
+        # ['id', 'name', 'title', 'version', 'url', 'author', 'author_email', 'maintainer', 'maintainer_email', 'notes', 'license_id', 'state']
+        fields = Package.revisioned_fields()
+        if not core_only:
+            fields += ['resources', 'tags', 'groups', 'extras', 'relationships']
+
+        if fields_to_ignore:
+            for field in fields_to_ignore:
+                fields.remove(field)
+
+        return fields
 
 def get_revisioned_classes_related_to_package():
     import resource
-    import extras
+    import package_extra
     import tag
     return [tag.PackageTag, resource.PackageResource,
-            extras.PackageExtra]
+            package_extra.PackageExtra]
 
