@@ -15,9 +15,8 @@ class TestGroupFieldset(PylonsTestCase):
 
     def test_1(self):
         group = model.Group.by_name(u'roger')
-        fs = ckan.forms.get_group_fieldset('group_fs').bind(group)
+        fs = ckan.forms.get_group_fieldset().bind(group)
         out = fs.render()
-        print out
         desc = fs.description.render()
         assert 'textarea' in desc, desc
         # TODO: ...
