@@ -2,13 +2,19 @@ from ckan.tests.pylons_controller import PylonsTestCase
 import ckan.model as model
 import ckan.forms
 from ckan.tests import *
+from ckan.tests.html_check import HtmlCheckMethods
 from ckan.lib.create_test_data import CreateTestData
 from pylons import config
 
 def _get_blank_param_dict(pkg=None, fs=None):
-    return ckan.forms.get_package_dict(pkg, blank=True, fs=fs)
+    return ckan.forms.get_package_dict(pkg, blank=True, fs=fs, user_editable_groups=[])
 
-class TestForm(PylonsTestCase):
+def get_fieldset(**kwargs):
+    if not kwargs.has_key('user_editable_groups'):
+        kwargs['user_editable_groups'] = []
+    return ckan.forms.get_gov_fieldset(**kwargs)
+
+class TestForm(PylonsTestCase, HtmlCheckMethods):
     @classmethod
     def setup_class(self):
         model.Session.remove()
@@ -19,8 +25,9 @@ class TestForm(PylonsTestCase):
         model.Session.remove()
         model.repo.rebuild_db()
 
+       
     def test_0_field_names(self):
-        fs = ckan.forms.get_gov_fieldset()
+        fs = get_fieldset()
         pkg = model.Package.by_name(u'private-fostering-england-2009')
         fs = fs.bind(pkg)
         out = fs.render()
@@ -32,7 +39,7 @@ class TestForm(PylonsTestCase):
         assert 'External reference' in out, out
 
     def test_1_field_values(self):
-        fs = ckan.forms.get_gov_fieldset()
+        fs = get_fieldset()
         pkg = model.Package.by_name(u'private-fostering-england-2009')
         fs = fs.bind(pkg)
         out = fs.render()
@@ -91,7 +98,7 @@ class TestForm(PylonsTestCase):
         self.check_tag(fs.temporal_coverage.render(), 'temporal_coverage-to', 'value="6/2009"')
 
     def test_2_field_department_selected(self):
-        fs = ckan.forms.get_gov_fieldset()
+        fs = get_fieldset()
         pkg = model.Package.by_name(u'private-fostering-england-2009')
         fs = fs.bind(pkg)
 
@@ -101,7 +108,7 @@ class TestForm(PylonsTestCase):
         assert 'option value="other">' in dept, dept
         assert 'Other:' in dept, dept
         assert 'value=""' in dept, dept
-        assert 'Department:</strong> Department for Children, Schools and Families' in fs.department.render_readonly(), fs.department.render_readonly()
+        assert 'Department for Children, Schools and Families' in fs.department.render_readonly(), fs.department.render_readonly()
 
     def test_2_field_department_none(self):
         # Create package
@@ -111,7 +118,7 @@ class TestForm(PylonsTestCase):
         model.repo.commit_and_remove()
 
         pkg = model.Package.by_name(u'test3')
-        fs = ckan.forms.get_gov_fieldset()
+        fs = get_fieldset()
         fs = fs.bind(pkg)
         out = fs.render()
         assert out
@@ -119,7 +126,7 @@ class TestForm(PylonsTestCase):
         dept_readonly = fs.department.render_readonly()
         assert '<select' in dept, dept
         assert '<option selected="selected" value=""></option>' in dept, dept
-        assert 'Department:</strong> <br/>' in dept_readonly, dept_readonly
+        assert '<p></p>' == dept_readonly, repr(dept_readonly)
 
     def test_2_field_department_other(self):
         # Create package
@@ -130,9 +137,10 @@ class TestForm(PylonsTestCase):
         model.repo.commit_and_remove()
 
         pkg = model.Package.by_name(u'test2')
-        fs = ckan.forms.get_gov_fieldset()
+        fs = get_fieldset()
         fs = fs.bind(pkg)
-        out = fs.render()
+        from pylons import c
+        out = fs.render(client=c)
         assert out
         dept = fs.department.render()
         dept_readonly = fs.department.render_readonly()
@@ -141,12 +149,17 @@ class TestForm(PylonsTestCase):
         self.check_tag(dept, 'option', 'value="other"', 'selected')
         assert 'Other:' in dept, dept
         assert 'value="Not on the list"' in dept, dept
-        assert 'Department:</strong> Not on the list' in dept_readonly, dept_readonly
+        assert 'Not on the list' in dept_readonly, dept_readonly
         
+    def test_3_restrict(self):
+        fs = get_fieldset(restrict=1)
+        restricted_fields = ('name', 'department', 'national_statistic')
+        for field_name in restricted_fields:
+            assert getattr(fs, field_name)._readonly, getattr(fs, field_name)
         
     def test_3_sync_new(self):
         newtagname = 'newtagname'
-        indict = _get_blank_param_dict(fs=ckan.forms.get_gov_fieldset())
+        indict = _get_blank_param_dict(fs=get_fieldset())
         prefix = 'Package--'
         indict[prefix + 'name'] = u'testname'
         indict[prefix + 'title'] = u'testtitle'
@@ -171,7 +184,7 @@ class TestForm(PylonsTestCase):
         indict[prefix + 'resources-0-url'] = u'http:/1'
         indict[prefix + 'resources-0-format'] = u'xml'
         indict[prefix + 'resources-0-description'] = u'test desc'
-        fs = ckan.forms.get_gov_fieldset().bind(model.Package, data=indict, session=model.Session)
+        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)
 
         model.repo.new_revision()
         fs.sync()
@@ -250,7 +263,7 @@ class TestForm(PylonsTestCase):
         assert pkg
 
         # edit it with form parameters
-        indict = _get_blank_param_dict(pkg=pkg, fs=ckan.forms.get_gov_fieldset())
+        indict = _get_blank_param_dict(pkg=pkg, fs=get_fieldset())
         prefix = 'Package-%s-' % pkg.id
         indict[prefix + 'name'] = u'testname2'
         indict[prefix + 'notes'] = u'some new notes'
@@ -274,7 +287,7 @@ class TestForm(PylonsTestCase):
         indict[prefix + 'resources-0-url'] = u'http:/1'
         indict[prefix + 'resources-0-format'] = u'xml'
         indict[prefix + 'resources-0-description'] = u'test desc'
-        fs = ckan.forms.get_gov_fieldset().bind(pkg, data=indict)
+        fs = get_fieldset().bind(pkg, data=indict)
 
         model.repo.new_revision()
         fs.sync()
@@ -324,18 +337,69 @@ class TestForm(PylonsTestCase):
                  (reqd_extra_key, reqd_extra_value,
                   outpkg.extras[reqd_extra_key])
 
+    def test_6_sync_update_restrict(self):
+        # create initial package
+        pkg_name = u'test_sync_restrict'
+        init_data = [{
+            'name':pkg_name,
+            'title':'test_title',
+            'extras':{
+              'notes':'Original notes',
+              'national_statistic':'yes',
+              'department':'dosac',
+              },
+            }]
+        CreateTestData.create_arbitrary(init_data)
+        pkg = model.Package.by_name(pkg_name)
+        assert pkg
+
+        # edit it with form parameters
+        indict = _get_blank_param_dict(pkg=pkg, fs=get_fieldset(restrict=1))
+        prefix = 'Package-%s-' % pkg.id
+        indict[prefix + 'notes'] = u'some new notes'
+        # try changing restricted params anyway
+        new_name = u'testname4' 
+        indict[prefix + 'name'] = new_name
+        indict[prefix + 'department'] = u'testdept'
+        # don't supply national_statistic param at all
+        fs = get_fieldset(restrict=1).bind(pkg, data=indict)
+        CreateTestData.flag_for_deletion(new_name)
+        
+        model.repo.new_revision()
+        fs.sync()
+        model.repo.commit_and_remove()
+
+        assert not model.Package.by_name(new_name) # unchanged
+        outpkg = model.Package.by_name(pkg_name) # unchanged
+        assert outpkg
+        # test sync worked
+        assert outpkg.notes == indict[prefix + 'notes']
+
+        # test gov fields
+        extra_keys = outpkg.extras.keys()
+        reqd_extras = {
+            'national_statistic':'yes', # unchanged
+            'department':init_data[0]['extras']['department'], # unchanged
+            }
+        for reqd_extra_key, reqd_extra_value in reqd_extras.items():
+            assert reqd_extra_key in extra_keys, 'Key "%s" not found in extras %r' % (reqd_extra_key, extra_keys)
+            assert outpkg.extras[reqd_extra_key] == reqd_extra_value, \
+                 'Extra %s should equal %s but equals %s' % \
+                 (reqd_extra_key, reqd_extra_value,
+                  outpkg.extras[reqd_extra_key])
+
     def test_5_validate_bad_date(self):
         # bad dates must be picked up in validation
-        indict = _get_blank_param_dict(fs=ckan.forms.get_gov_fieldset())
+        indict = _get_blank_param_dict(fs=get_fieldset())
         prefix = 'Package--'
         indict[prefix + 'name'] = u'testname3'
         indict[prefix + 'date_released'] = u'27/11/2008'
-        fs = ckan.forms.get_gov_fieldset().bind(model.Package, data=indict, session=model.Session)
+        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)
         validation = fs.validate()
         assert validation
 
         indict[prefix + 'date_released'] = u'27/11/0208'
-        fs = ckan.forms.get_gov_fieldset().bind(model.Package, data=indict, session=model.Session)
+        fs = get_fieldset().bind(model.Package, data=indict, session=model.Session)
         validation = fs.validate()
         assert not validation
 
