@@ -49,6 +49,7 @@ class TestCreation(object):
 
         # test delete-orphan
         q = model.Session.query(model.UserObjectRole)
+        q = q.filter_by(user=mradmin)
         assert q.count() == 2, q.all()
         uow = q.filter_by(context=u'user_object').first()
         uow.user = None
@@ -75,7 +76,6 @@ class TestCreation(object):
         anna = model.Package.by_name(u'annakarenina')
         mradmin = model.User.by_name(u'tester')
         roles = self.authorizer.get_roles(mradmin.name, anna)
-        print model.Session.query(model.PackageRole).filter_by(user=mradmin).all()
         assert role in roles, roles
 
     def test_2_role_action_basic(self):
@@ -124,8 +124,10 @@ class TestDefaultRoles(object):
         assert self.is_allowed(model.Role.EDITOR, model.Action.EDIT)
 
     def test_create(self):
-        assert self.is_allowed(model.Role.READER, model.Action.CREATE)
-        assert self.is_allowed(model.Role.EDITOR, model.Action.CREATE)
+        assert self.is_allowed(model.Role.READER, model.Action.PACKAGE_CREATE)
+        assert self.is_allowed(model.Role.EDITOR, model.Action.PACKAGE_CREATE)
+        assert not self.is_allowed(model.Role.READER, model.Action.GROUP_CREATE)
+        assert self.is_allowed(model.Role.EDITOR, model.Action.GROUP_CREATE)
 
     def test_edit_permissions(self):
         assert not self.is_allowed(model.Role.READER, model.Action.EDIT_PERMISSIONS)
@@ -242,7 +244,6 @@ class TestUsage(object):
         anna = model.Package.by_name(u'annakarenina')
         tester = model.User.by_name(u'tester')
         roles = self.authorizer.get_roles(tester.name, anna)
-        print model.Session.query(model.PackageRole).filter_by(user=tester).all()
         assert self.admin_role in roles, roles
 
     def test_2_is_auth_admin(self):
